﻿using System;
using System.Diagnostics;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using TraceLabSDK;
using TraceLabSDK.Component.Config;
using TraceLabSDK.Types;
using BulletsParser_TraceLabComponent;
using SectionSplitter;
using SentenceParser;

namespace CombinedPolicyParser
{
    [Component(Name = "Combined Policy Parser",
                Description = "Parse by heading/paragraph; parse by sentence; parse by bullets",
                Author = "Satrio Husodo",
                Version = "1.0",
                ConfigurationType = typeof(CombinedParserConfig)
                )]
    [IOSpec(IOSpecType.Output, "outputName", typeof(TLArtifactsCollection))]
    public class CombinedPolicyParser : BaseComponent
    {
        private CombinedParserConfig config;
        public CombinedPolicyParser(ComponentLogger log) : base(log) 
        {
            config = new CombinedParserConfig();
            Configuration = config;
        }
        
        public override void Compute()
        {
            string delimiter = config.delimiter;
            String inputfile = @config.Artifacts.Absolute;
            string[] lines = File.ReadAllLines(@inputfile);
            String out_dir = @config.OutDir.Absolute;
            List<string> items = new List<string>();
            if (config.parse_options == ParseOptions.HeadingAndSection)
            {
                items = SectionSplitter.SectionSplitter.get_items(lines);
            }
            else if (config.parse_options == ParseOptions.BulletedList){
                items = BulletsParser_TL.get_items(lines, config.bullet_type_level1, config.bullet_type_level2, delimiter);
            }
            else if (config.parse_options == ParseOptions.Sentence)
            {
                
                items = SentenceParser.SectionSplitter.get_items(lines);
            }
            TLArtifactsCollection TL_items = new TLArtifactsCollection();
            int i = 0;
            foreach (string s in items)
            {
                i = i + 1;
                TL_items.Add(new TLArtifact(Convert.ToString(i), s));
                string out_file = @out_dir + @"\S" + i + ".txt";
                System.IO.File.WriteAllText(out_file, s);
            }
            Workspace.Store("outputName", TL_items);
        }
    }

    public enum ParseOptions
        {
            HeadingAndSection,
            Sentence,
            BulletedList
        }
    public class CombinedParserConfig 
    {
        [DisplayName("Parse by")]
        [Description("Choose among 3 parsing options")]
        public ParseOptions parse_options { get; set; } 
        [DisplayName("Artifacts location")]
        [Description("Location of artifacts file")]
        public FilePath Artifacts { get; set; }
        [DisplayName("Delimiter")]
        [Description("Set delimiter between number/bullet and text. e.g. '.' or ','")]
        public string delimiter { get; set; }
        [DisplayName("Level 1 bullet format")]
        [Description("Set level 1 bulleting format as 'numbers', 'letters', or 'roman numerals'")]
        public Bullet_type bullet_type_level1 { get; set; }
        [DisplayName("Level 2 bullet format")]
        [Description("Set level 2 bulleting format as 'numbers', 'letters', or 'roman numerals'")]
        public Bullet_type bullet_type_level2 { get; set; }
        [DisplayName("Output Directory")]
        [Description("Directory for output files")]
        public DirectoryPath OutDir { get; set; }
    }
}